#pragma once

#include <stdexcept>
#include <vector>

// A weighted grade consists of the raw grade and a weight that determines
// its value relative to other grades.
struct Weighted_grade
{
    double grade;
    double weight;
};

// Thrown when the average is undefined because the total weight is 0.
struct average_undefined : std::runtime_error {
    average_undefined() : runtime_error("Average undefined") { }
};

// Thrown when the input doesn't make sense because it contains a negative
// weight.
struct negative_weight : std::runtime_error {
    negative_weight() : runtime_error("Negative weight") { }
};

// The result of computing which grade to omit: the best average and the
// index of the omitted grade.
struct Omission_result
{
    double average;
    int index;
};

// Reads a vector of weighted grades from cin, stopping when it encounters a
// non-number of EOF. Each weighted grade is read as the raw grade followed
// by the weight.
std::vector<Weighted_grade> read_grades();

// Computes the weighted sum of a vector with the indicated element omitted.
// Throws negative_weight if a negative weight is encountered, since that
// doesn't make sense for a weighted sum.
double weighted_sum_without(std::vector<Weighted_grade> grades,
                            int which);

// Computes the sum of the weights of a vector with the indicated element
// omitted.
double total_weight_without(std::vector<Weighted_grade> grades, int which);

// Computes weighted average; exits with error message if total weight is 0.
double weighted_average_without(std::vector<Weighted_grade> grades, int which);

// Finds the best grade to omit and the resulting best average.
Omission_result find_best_omission(std::vector<Weighted_grade> grades);

// Prints the omission result: the best average followed by the omitted index.
void print_omission_result(Omission_result result);
